@@ -26,6 +26,10 @@ namespace SIMDUTF_IMPLEMENTATION {
 #include "rvv/rvv_utf32_to.inl.cpp"
 #include "rvv/rvv_utf8_to.inl.cpp"
 
+#if SIMDUTF_FEATURE_BASE64
+  #include "rvv/rvv_find.cpp"
+#endif
+
 #if SIMDUTF_FEATURE_DETECT_ENCODING
 simdutf_warn_unused int
 implementation::detect_encodings(const char *input,
@@ -118,6 +122,16 @@ size_t implementation::binary_to_base64(const char *input, size_t length,
                                         base64_options options) const noexcept {
   return scalar::base64::tail_encode_base64(output, input, length, options);
 }
+
+const char *implementation::find(const char *start, const char *end,
+                                 char character) const noexcept {
+  return util_find(start, end, character);
+}
+
+const char16_t *implementation::find(const char16_t *start, const char16_t *end,
+                                     char16_t character) const noexcept {
+  return util_find(start, end, character);
+}
 #endif // SIMDUTF_FEATURE_BASE64
 
 } // namespace SIMDUTF_IMPLEMENTATION
