@@ -1,6 +1,7 @@
 #ifndef SIMDUTF_BASE64_H
 #define SIMDUTF_BASE64_H
 
+#include <algorithm>
 #include <cstddef>
 #include <cstdint>
 #include <cstring>
@@ -117,6 +118,18 @@ reduced_input find_end(const char_type *src, size_t srclen,
   size_t full_input_length = srclen;
   size_t equallocation =
       srclen; // location of the first padding character if any
+  if (ignore_garbage) {
+    // Technically, we don't need to find the first padding character, we can
+    // just change our algorithms, but it adds substantial complexity.
+    auto it = simdutf::find(src, src + srclen, '=');
+    if (it != src + srclen) {
+      equallocation = it - src;
+      equalsigns = 1;
+      srclen = equallocation;
+      full_input_length = equallocation + 1;
+    }
+    return {equalsigns, equallocation, srclen, full_input_length};
+  }
   if (!ignore_garbage && srclen > 0 && src[srclen - 1] == '=') {
     // This is the last '=' sign.
     equallocation = srclen - 1;
@@ -272,15 +285,16 @@ full_result base64_tail_decode_impl(
           return {BASE64_INPUT_REMAINDER, size_t(src - srcinit),
                   size_t(dst - dstinit), true};
         } else
-          // The idea here is that in stop_before_partial mode, we stop right
-          // away if we have a partial chunk that would be valid.
-          if (last_chunk_options ==
-                  last_chunk_handling_options::stop_before_partial &&
-              (idx >= 2 ||
-               padding_characters ==
-                   0)) { // There are two cases where stop_before_partial
-                         // succeeds: either we have idx >= 2 or we have
-                         // padding_characters == 0
+          // If there is a partial chunk with insufficient padding, with
+          // stop_before_partial, we need to just ignore it. In "only full"
+          // mode, skip the minute there are padding characters.
+          if ((last_chunk_options ==
+                   last_chunk_handling_options::stop_before_partial &&
+               (padding_characters + idx < 4) &&
+               (idx >= 2 || padding_characters == 0)) ||
+              (last_chunk_options ==
+                   last_chunk_handling_options::only_full_chunks &&
+               (idx >= 2 || padding_characters == 0))) {
             // Rewind src to before partial chunk
 
             // partial means that we are *not* going to consume the read
@@ -342,10 +356,8 @@ full_result base64_tail_decode_impl(
               }
               dst += 2;
             } else if (!ignore_garbage && idx == 1 &&
-                       (last_chunk_options !=
-                            last_chunk_handling_options::stop_before_partial ||
-                        (last_chunk_options ==
-                             last_chunk_handling_options::stop_before_partial &&
+                       (!is_partial(last_chunk_options) ||
+                        (is_partial(last_chunk_options) &&
                          padding_characters > 0))) {
               return {BASE64_INPUT_REMAINDER, size_t(src - srcinit),
                       size_t(dst - dstinit)};
@@ -411,11 +423,14 @@ patch_tail_result(full_result r, size_t previous_input, size_t previous_output,
     r.input_count = equallocation;
   }
 
-  if (r.error == error_code::SUCCESS &&
-      last_chunk_options != stop_before_partial) {
-    // A success when we are not in stop_before_partial mode.
-    // means that we have consumed the whole input buffer.
-    r.input_count = full_input_length;
+  if (r.error == error_code::SUCCESS) {
+    if (!is_partial(last_chunk_options)) {
+      // A success when we are not in stop_before_partial mode.
+      // means that we have consumed the whole input buffer.
+      r.input_count = full_input_length;
+    } else if (r.output_count % 3 != 0) {
+      r.input_count = full_input_length;
+    }
   }
   return r;
 }
@@ -526,8 +541,8 @@ simdutf_warn_unused full_result base64_to_binary_details_impl(
       output, input, length, equalsigns, options, last_chunk_options);
   r = scalar::base64::patch_tail_result(r, 0, 0, equallocation,
                                         full_input_length, last_chunk_options);
-  if (last_chunk_options != stop_before_partial &&
-      r.error == error_code::SUCCESS && equalsigns > 0 && !ignore_garbage) {
+  if (!is_partial(last_chunk_options) && r.error == error_code::SUCCESS &&
+      equalsigns > 0 && !ignore_garbage) {
     // additional checks
     if ((r.output_count % 3 == 0) ||
         ((r.output_count % 3) + 1 + equalsigns != 4)) {
@@ -562,8 +577,8 @@ simdutf_warn_unused full_result base64_to_binary_details_safe_impl(
       output, outlen, input, length, equalsigns, options, last_chunk_options);
   r = scalar::base64::patch_tail_result(r, 0, 0, equallocation,
                                         full_input_length, last_chunk_options);
-  if (last_chunk_options != stop_before_partial &&
-      r.error == error_code::SUCCESS && equalsigns > 0 && !ignore_garbage) {
+  if (!is_partial(last_chunk_options) && r.error == error_code::SUCCESS &&
+      equalsigns > 0 && !ignore_garbage) {
     // additional checks
     if ((r.output_count % 3 == 0) ||
         ((r.output_count % 3) + 1 + equalsigns != 4)) {
